package io.reisub.openosrs.autoalcher;

import com.google.inject.Provides;
import io.reisub.openosrs.util.Calculations;
import io.reisub.openosrs.util.Util;
import lombok.extern.slf4j.Slf4j;
import net.runelite.api.ItemID;
import net.runelite.api.Varbits;
import net.runelite.api.events.GameTick;
import net.runelite.api.events.ItemContainerChanged;
import net.runelite.client.config.ConfigManager;
import net.runelite.client.eventbus.Subscribe;
import net.runelite.client.events.ConfigChanged;
import net.runelite.client.game.RunepouchRune;
import net.runelite.client.input.KeyListener;
import net.runelite.client.input.KeyManager;
import net.runelite.client.plugins.Plugin;
import net.runelite.client.plugins.PluginDependency;
import net.runelite.client.plugins.PluginDescriptor;
import net.runelite.client.plugins.iutils.api.Spells;
import net.runelite.client.plugins.iutils.game.Game;
import net.runelite.client.plugins.iutils.game.InventoryItem;
import net.runelite.client.plugins.iutils.game.iWidget;
import net.runelite.client.plugins.iutils.iUtils;
import org.pf4j.Extension;

import javax.inject.Inject;
import java.awt.event.KeyEvent;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

@Extension
@PluginDependency(Util.class)
@PluginDependency(iUtils.class)
@PluginDescriptor(
	name = "Chaos Auto Alcher",
	description = "Gold! Always believe in your soul.",
	enabledByDefault = false
)
@Slf4j
public class Autoalcher extends Plugin implements KeyListener {
	@Inject
	private Game game;

	@Inject
	private Config config;

	@Inject
	private Calculations calc;

	@Inject
	private KeyManager keyManager;

	@Provides
	Config provideConfig(ConfigManager configManager) {
		return configManager.getConfig(Config.class);
	}

	private static final Varbits[] AMOUNT_VARBITS = {
			Varbits.RUNE_POUCH_AMOUNT1, Varbits.RUNE_POUCH_AMOUNT2, Varbits.RUNE_POUCH_AMOUNT3
	};
	private static final Varbits[] RUNE_VARBITS = {
			Varbits.RUNE_POUCH_RUNE1, Varbits.RUNE_POUCH_RUNE2, Varbits.RUNE_POUCH_RUNE3
	};

	private ScheduledExecutorService executor;
	private boolean active;
	private long last;
	private String[] names;

	@Override
	protected void startUp() {
		log.info("Starting Chaos Auto Alcher");

		executor = Executors.newSingleThreadScheduledExecutor();
		keyManager.registerKeyListener(this);
	}

	@Override
	protected void shutDown() {
		log.info("Stopping Chaos Auto Alcher");

		executor.shutdownNow();
		keyManager.unregisterKeyListener(this);
	}

	@Subscribe
	private void onGameTick(GameTick event) {
		if (!active || last + 5 > game.ticks()) return;

		if (names == null) parseItems();

		if (names == null || names.length == 0) return;

		if (game.inventory().withNamePart(names).exists()
				&& hasRunes()) {
			executor.schedule(this::alch, calc.random(100, 200), TimeUnit.MILLISECONDS);
		}
	}

	@Subscribe
	private void onConfigChanged(ConfigChanged event) {
		if (!event.getGroup().equals("chaosautoalcher")) return;

		if (event.getKey().equals("alchItems")) {
			parseItems();
		}
	}

	private void alch() {
		InventoryItem item = game.inventory().withNamePart(names).first();
		iWidget spellWidget = game.widget(Spells.HIGH_LEVEL_ALCHEMY.getInfo());
		if (item == null || spellWidget == null) return;

		spellWidget.useOn(item);
		last = game.ticks();

		game.tick(3);
		game.openInterface(3);
	}

	private void parseItems() {
		names = config.alchItems().split(";");

		for (int i = 0; i < names.length; i++) {
			names[i] = names[i].trim();
		}
	}

	@Override
	public void keyTyped(KeyEvent e) {

	}

	@Override
	public void keyPressed(KeyEvent e) {
		if (config.autoalchHotkey().matches(e)) {
			active = !active;

			if (active) {
				game.utils.sendGameMessage("Enabled Auto Alcher");
			} else {
				game.utils.sendGameMessage("Disabled Auto Alcher");
			}
		}
	}

	@Override
	public void keyReleased(KeyEvent e) {

	}

	private boolean hasRunes() {
		int fireRunes = (int) game.inventory().withId(ItemID.FIRE_RUNE).count();
		int natureRunes = (int) game.inventory().withId(ItemID.NATURE_RUNE).count();

		if (game.inventory().withId(ItemID.RUNE_POUCH).exists()) {
			for (int i = 0; i < AMOUNT_VARBITS.length; i++) {
				int amount = game.client().getVar(AMOUNT_VARBITS[i]);
				if (amount <= 0) continue;

				RunepouchRune rune = RunepouchRune.getRune(game.client().getVar(RUNE_VARBITS[i]));

				switch (rune) {
					case NATURE:
						natureRunes += amount;
						break;
					case FIRE:
						fireRunes += amount;
						break;
				}
			}
		}

		return fireRunes >= 5 && natureRunes >= 1;
	}
}